; JCM-1 PONG
; Version 0.2
; Written by Joe Strosnider
; inanis.net
; youtube.com/user/umbravector

.target "65C02"
.include "JCM-1_BIOS.inc"

LeftPos  	= $80
RightPos	= $81
BallXPos	= $82
BallYPos	= $83
BallXDir    = $84
BallYDir	= $85
LeftScore	= $86
RightScore  = $87
WhoScored   = $88
RegW		= $89
RegZ		= $8A
PixelY		= $8B
RestPeriod  = $8C
PadToCheck	= $90
JoyPadValue = $91
PosToCheck  = $92

			*=$0300

				; Set the sound chip I/O ports to input.
SetupIO			LDA #1
				STA Sound_ChipSelect	; We're talking to Sound Chip 1
				
				LDA #7
				STA Sound_Register	; We're setting up register 7
				
				LDA #%00111111		; this means "input on channel B and A"
				STA Sound_Value		; set up the input
				JSR B_SoundOut		; make it so.
				
StartGame		JSR InitGame
				JMP StartOver1

GetInput	
	; get left player input
				LDA #14				; Pad A or Left Paddle
				STA PadToCheck
				JSR GetJoyPad

	; Get right player input
				LDA #15				; Pad B or Right Paddle
				STA PadToCheck
				JSR GetJoyPad

MoveBall
	; JMP CheckState ;diagnostics - this will let you diagnose paddle movement.
    ; this code is crufty but works. I'm sure there's some sort of 6502 binary math magic to make it better.
	; also, as you can see, the ball only goes in 45 degree angles. pretty cruddy, but it will work for a demo.

				JSR EraseBall
	CheckBallX	LDA BallXDir		; horizontal direction
				BEQ AddBallX		; if zero, it's going right. Increment the X position
				
	SubBallX	DEC BallXPos		; if nonzero, it's going left. Decrement the X position
				BRA CheckBallY		;

	AddBallX	INC BallXPos

	CheckBallY	LDA BallYDir		; vertical direction
				BEQ AddBallY		; if zero it's going down, go add to the position
	SubBallY	DEC BallYPos		; if nonzero, it's going up. Go subtract from the position
				BRA DrawBall1
	AddBallY	INC BallYPos	

DrawBall1		JSR DrawBall

CheckState		LDA BallXPos		; check the Ball's X position
				CMP #2				; Compare with left border
				BEQ	CheckLPos		; Hit the left border. Go check against left paddle position
				CMP #79				; compare against right border
				BEQ	CheckRPos		; It hit the right border. Go check against right paddle position.
				LDA BallYPos		; Check the Ball's Y position
				CMP #1				; Compare with top of screen
				BEQ BounceDown		; It's at the top, bounce off top of screen (downward)
				CMP #23				; Compare with bottom of screen
				BEQ	BounceUp		; it's 23. bounce off bottom of screen. (upward)
				JMP StartAgain		; no bounces happened. Loop again.

	BounceDown	STZ BallYDir
				BRA BncCont1
				
	BounceUp	LDA #128
				STA BallYDir
	BncCont1	JSR B_Bell
				JMP StartAgain
	
	CheckLPos	; is the left paddle in the ball scope?
				; yes = bounce right
				; no = right scores
				
				LDA LeftPos
				CMP BallYPos		; Check to see if it's exactly equal
				BEQ BounceRight 	; it's exactly the same, so it bounces.
				BCS RightScored		; We've already checked equal so if this trips
									;    then the paddle top has a higher value (i.e lower on the screen) than the ball.
				ADC #6				; Add 6 so we can compare to the bottom
				CMP BallYPos    	; Check to see if it's below the paddle
				BCC RightScored 	; if this is less than the ball, that means the paddle is higher on the screen then the ball
				BRA BounceRight 	; passed both checks, bounce the ball
				
	CheckRPos	; is the right paddle in the ball scope?
				; yes = bounce left
				; no = left scores
				
				LDA RightPos
				CMP BallYPos		; Check to see if it's exactly equal
				BEQ BounceLeft  	; it's exactly the same, so it bounces.
				BCS LeftScored		; We've already checked equal so if this trips
									; then the paddle top has a higher value (i.e lower on the screen) than the ball.
				ADC #6				; Add 6 so we can compare to the bottom
				CMP BallYPos    	; Check to see if it's below the paddle
				BCC LeftScored  	; if this is less than the ball, that means the paddle is higher on the screen then the ball
									; passed both checks, bounce the ball
	BounceLeft	LDA #128
				STA BallXDir
	BncCont2	JSR B_Bell
				BRA StartAgain
				
	BounceRight	STZ BallXDir
				BRA BncCont2
				
	
	RightScored	INC RightScore
				LDA #32				; 32 = last person who scored was right.
				BRA ScrChk1
	LeftScored	INC LeftScore
				LDA #64				; 64 = last person who scored was left
	ScrChk1		STA WhoScored
	WinCheck	LDA LeftScore		; Check to see if anyone won
				CMP #10
				BEQ LeftWon
				LDA RightScore
				CMP #10
				BEQ RightWon	
				
    ContCheck	LDA WhoScored
				CMP #64				; 64 = Left
				BEQ PtLeft
	PtRight		JSR B_PrintImm
				.byte $1B,"[11;34HRight Scored!",$00	
				BRA ContScoring
				
	PtLeft		JSR B_PrintImm
				.byte $1B,"[11;34HLeft Scored!",$00

	ContScoring	STZ WhoScored		; someone scored!
				JSR	B_BootChime		; play score sound
				JSR Whole			; pause a while
	StartOver1	JSR DrawReady		; Draw Get Ready Message
				JSR ResetBoard		; reset the paddles and ball
				JSR Whole			; pause a while
				JSR EraseReady
	
StartAgain		LDA #$03			; master
				STA RestPeriod		; ...timing
				JSR Whole			; ......loop - makes the game playable
				LDA #$60
				STA RestPeriod
				JMP GetInput

LeftWon			JSR B_PrintImm
				.byte $1B,"[2J",$1B,"[11;33HLeft Won PONG!",$00
				BRA PlayAgain
				
RightWon		JSR B_PrintImm
				.byte $1B,"[2J",$1B,"[11;33HRight Won PONG!",$00				

PlayAgain		JSR B_PrintImm
				.byte $1B,"[12;33HPlay Again? (Y/N) ",$00
	PAInput		JSR B_CharIn
				AND #$DF			; Convert to uppercase.
				CMP #"Y"			; Yes?
				BEQ Go				; Go again.
				CMP #"N"			; No?
				BEQ Stop
				BRA PAInput
				
Go				JMP StartGame
Stop			JMP Exit

EraseReady		JSR B_PrintImm
				.byte $1B,"[14;35H          ",$00 ;
				RTS
			
DrawReady		JSR B_PrintImm
				.byte $1B,"[2J",$1B,"[14;35HGet Ready!",$00 ;
				RTS

InitGame		;Set up game start values
				LDA #$60
				STA RestPeriod
				STZ LeftScore
				STZ RightScore
				STZ WhoScored
ResetBoard		;Set up basic board values
				STZ BallXDir		; Ball starts by going to the right and up.
				STZ BallYDir		; 0 = right/down. 128= left/up.
				
				LDA #1
				STA LeftPos			; Left paddle is at top left.
							
				LDA #40
				STA BallXPos		; Ball is off center for testing
				LDA #12
				STA BallYPos

				LDA #18
				STA RightPos		; Right Paddle is at bottom right.
				JSR DrawPaddles
								
				JSR B_PrintImm		; draw message
				.byte $1B,"[24;14HJCM PONG!     -= Press Esc to Exit =-     !GNOP MCJ",$00 ;

				;draw score
				JSR B_PrintImm
				.byte $1B,"[24;2H",$00
				LDA LeftScore
				JSR OUTDEC8
				
				JSR B_PrintImm
				.byte $1B,"[24;78H",$00
				LDA RightScore
				JSR OUTDEC8				
				RTS	
				
Sixteenth		LDX #0
				LDY #0
WaitA 			INX
				BNE WaitA
				JSR QuitChk
				INY
				CPY RestPeriod			; Master Timing Value!!
				BNE WaitA
				RTS
Whole			LDA #16
RepeatRest		JSR Sixteenth
				DEC
				BNE RepeatRest
				RTS


; From http://6502org.wikidot.com/software-output-decimal		
; Prints an 8-bit value as an ASCII number to the serial terminal
; Enter routine with value to convert in A.
; Destroys A,X,Y
;		
OUTDEC8			LDX #1
				STX DataC
				INX
				LDY #$40
		Step1	STY DataB
				LSR
		Step2	ROL
				BCS Step3
				CMP DataA,X
				BCC Step4
		Step3	SBC DataA,X
				SEC
		Step4	ROL DataB
				BCC Step2
				TAY
				CPX DataC
				LDA DataB
				BCC Step5
				BEQ Step6
				STX DataC
		Step5	EOR #$30
				JSR B_ACIA0_Out
		Step6	TYA
				LDY #$10
				DEX
				BPL Step1
				RTS

		DataA 	.byte 128,160,200
		DataB 	.byte 0
		DataC 	.byte 0

QuitChk			PHA						; save A cause it's in use
				LDA 	ACIA0_Status	; Get ACIA Status.
				AND 	#$08 			; Was RX buffer empty?
				BEQ 	KeepPlaying		; Yes. Keep playing
		Exit	LDA 	ACIA0_Data		; No. Bomb out. Clear ACIA of garbage.
				JSR 	B_SoundClear	; Kill sound and joypad input.
				JSR 	B_PrintImm
				.byte 	$1B,"[2J",$00	; Clear screen.
				PLA     ;clear the jsr that got us here
				PLA     ;from the stack
				JMP 	M_Prompt		; Go back to the prompt.
	KeepPlaying	PLA						; Restore A
				RTS

DrawPaddles
				LDA #1
				STA RegW
				LDA LeftPos
		PLoop	STA PixelY
				STZ RegZ
				
				PDLoop	JSR ANSIHead
						LDA PixelY
						ADC RegZ
						JSR OUTDEC8
						LDA #";"
						JSR B_ACIA0_Out
						LDA RegW
						JSR OUTDEC8
						JSR B_PrintImm
						.byte "H",178,$00	; Separator, xpos, end char, paddle pixel.
						INC RegZ
						LDA RegZ
						CMP #7
						BNE PDLoop
				
				LDA RegW
				CMP #1
				BNE PaddleRTS
				LDA #80
				STA RegW
				LDA RightPos
				BRA PLoop
	PaddleRTS	RTS


EraseBall		LDA #" "
				BRA DrawBall2
DrawBall		LDA #"o"
DrawBall2		STA RegW
				JSR ANSIHead
			
				LDA BallYPos		; 
				JSR OUTDEC8			; send out the Y position.
			
				LDA #";"			; load up the separator
				JSR B_ACIA0_Out		; send it out.
			
				LDA BallXPos		; 
				JSR OUTDEC8			; send out the y position
			
				LDA #"H"
				JSR B_ACIA0_Out
			
				LDA RegW
				JSR B_ACIA0_Out
				RTS

/* ******************************************************** */
; fetch and interpret player input
; joypad register to check must be in PadToCheck
; 14 = A
; 15 = B
			
GetJoyPad		STA Sound_Register	;
				JSR B_JoyIn			; Get It!
				STA JoyPadValue
				LDA PadToCheck
				CMP #14
				BEQ PadIs1
	PadIs2		LDA #80
				STA PadToCheck
				LDA RightPos
				BRA PadCont
	PadIs1		LDA #1
				STA PadToCheck
				LDA LeftPos
	PadCont		STA PosToCheck
			
				LDA JoyPadValue
					;bit 7 is in bit 7
				ASL ;bit 6 <
				ASL ;bit 5 <
				ASL ;bit 4 <
				ASL ;bit 3 <
				ASL ;bit 2 <
				ASL ;bit 1 <
				STA JoyPadValue
				BIT JoyPadValue
				BPL PadDown		    ; Down is pressed. Deal with it.
				ASL ;bit 0 <		; Down isn't pressed, so let's check for up.
				STA JoyPadValue
				BIT JoyPadValue
				BPL PadUp
	JPReturn	RTS					; nothing pressed. Move on

	; is down pressed? add one to position
	PadDown	    LDA PosToCheck
				CMP #18
				BEQ JPReturn		; already 18, nowhere to move

				STA PixelY			; erase top pixel
				JSR ANSIHead
				LDA PixelY
				JSR OUTDEC8
				JSR SendPadPos
				LDA #" "			; eraser.
				JSR B_ACIA0_Out
			
				LDA PixelY
				ADC #7 				; increment 7 (becase we haven't inc'd LeftPos Yet!)
				STA RegZ			; store for adding pixel
				JSR ANSIHead		; draw bottom pixel
				LDA RegZ
				JSR OUTDEC8
				JSR SendPadPos
				LDA #178			; pixel.
				JSR B_ACIA0_Out				
				INC PosToCheck 		; not already 18. make it one more.
				BRA SettlePos
				RTS
			
	; is up pressed? subtract 1 from position
	PadUp		LDA PosToCheck
				CMP #1
				BEQ JPReturn		; already one, do nothing
			
				STA PixelY			; draw top pixel
				JSR ANSIHead
				LDA PixelY
				SBC #1
				JSR OUTDEC8
				JSR SendPadPos
				LDA #178			; pixel.
				JSR B_ACIA0_Out				
			
				LDA PixelY
				ADC #6 				; increment 6 (becase we haven't dec'd LeftPos Yet!
				STA RegZ			; store for adding pixel
				JSR ANSIHead		; erase bottom pixel
				LDA RegZ
				JSR OUTDEC8
				JSR SendPadPos
				LDA #" "			; eraser.
				JSR B_ACIA0_Out				
				DEC PosToCheck		; not already one. make it one less.
			
			
	SettlePos	LDA PadToCheck
				CMP #1
				BEQ Settle1
		Settle2	LDA PosToCheck
				STA RightPos
				RTS
		Settle1	LDA PosToCheck
				STA LeftPos
				RTS
				
ANSIHead 		JSR B_PrintImm
				.byte $1B,"[",0
				RTS
			
SendPadPos		LDA #";"			; Separator
				JSR B_ACIA0_Out
				LDA PadToCheck		; Pad Postion
				JSR OUTDEC8
				LDA #"H"			; End Char
				JSR B_ACIA0_Out
				RTS

			
/* ******************************************************** */
B_JoyIn	; Reads input value from I/O Port on Sound Chip 1
		; Enter with Chip Register to read in Sound_Register
		; Assumes you've already set up the chip to Input
		; Returns with value in A
				LDA #$FF      	  		; $FF = Output
				STA Sound_DDRA			; Reg A Direction
				STA Sound_DDRB			; Reg B Direction
				LDA	Sound_Register		; Grab the Register we're working with.
				STA Sound_OutRegA		; Send it to the chip.
				LDA #12        			; Command value to strobe in the register address for chip 1 - 0000 1100
				STA Sound_OutRegB		; Turn on the command to the chip.
				STZ Sound_OutRegB		; Turn off the command to the chip.
				STZ Sound_DDRA			; Reg A Direction: 0=In
				LDA #4					; Read Command for Sound Chip 1 = READ 0000 0100
				STA Sound_OutRegB   	; Send the command to the chip.
				LDA Sound_OutRegA		; Get the data from the chip.
				STZ Sound_OutRegB		; Turn off the command to the chip.
				RTS